@@ -15,7 +15,7 @@ use canonical_derive::Canon;
 use crate::annotations::{Annotation, Combine};
 use crate::branch::Branch;
 use crate::branch_mut::BranchMut;
-use crate::compound::{Child, Compound, MutableLeaves};
+use crate::compound::{AnnoIter, Child, Compound, MutableLeaves};
 use crate::walk::{Step, Walk, Walker};
 
 /// The maximum value of a collection
@@ -81,32 +81,32 @@ where
 impl<K, L> Annotation<L> for MaxKey<K>
 where
     L: Keyed<K>,
-    K: Clone,
+    K: Clone + Ord + Canon,
 {
     fn from_leaf(leaf: &L) -> Self {
         MaxKey::Maximum(leaf.key().clone())
     }
 }
 
-impl<C, A, K> Combine<C, A> for MaxKey<K>
+impl<K, A> Combine<A> for MaxKey<K>
 where
-    C: Compound<A>,
-    C::Leaf: Keyed<K>,
-    A: Annotation<C::Leaf> + Borrow<Self>,
     K: Ord + Clone,
+    A: Borrow<Self> + Canon,
 {
-    fn combine(node: &C) -> Self {
-        let mut max = MaxKey::NegativeInfinity;
-
-        for child in node.children() {
-            let ann = &*child.annotation();
-            let m = ann.borrow();
-
+    fn combine<C>(iter: AnnoIter<C, A>) -> Self
+    where
+        C: Compound<A>,
+        A: Annotation<C::Leaf>,
+    {
+        iter.fold(MaxKey::NegativeInfinity, |max, ann| {
+            let m = (*ann).borrow();
+            // We only clone if neccesary
             if *m > max {
-                max = m.clone()
+                m.clone()
+            } else {
+                max
             }
-        }
-        max
+        })
     }
 }
 
@@ -123,7 +123,7 @@ impl<C, A, K> Walker<C, A> for FindMaxKey<K>
 where
     C: Compound<A>,
     C::Leaf: Keyed<K>,
-    A: Combine<C, A> + Borrow<MaxKey<K>>,
+    A: Annotation<C::Leaf> + Borrow<MaxKey<K>>,
     K: Ord + Clone + core::fmt::Debug,
 {
     fn walk(&mut self, walk: Walk<C, A>) -> Step {
@@ -141,9 +141,10 @@ where
                     }
                 }
                 Child::Node(n) => {
-                    let node_max = n.annotation().borrow().clone();
-                    if node_max > current_max {
-                        current_max = node_max;
+                    let ann = n.annotation();
+                    let node_max: &MaxKey<K> = (*ann).borrow();
+                    if node_max > &current_max {
+                        current_max = node_max.clone();
                         current_step = Step::Into(i);
                     }
                 }
@@ -161,7 +162,7 @@ pub trait GetMaxKey<'a, A, K>
 where
     Self: Compound<A>,
     Self::Leaf: Keyed<K>,
-    A: Combine<Self, A> + Borrow<MaxKey<K>> + Clone,
+    A: Annotation<Self::Leaf> + Borrow<MaxKey<K>>,
     K: Ord,
 {
     /// Construct a `Branch` pointing to the element with the largest key
@@ -172,7 +173,6 @@ where
         &'a mut self,
     ) -> Result<Option<BranchMut<'a, Self, A>>, CanonError>
     where
-        A: Combine<Self, A>,
         Self: MutableLeaves;
 }
 
@@ -180,7 +180,7 @@ impl<'a, C, A, K> GetMaxKey<'a, A, K> for C
 where
     C: Compound<A> + Clone,
     C::Leaf: Keyed<K>,
-    A: Combine<C, A> + Borrow<MaxKey<K>>,
+    A: Annotation<C::Leaf> + Borrow<MaxKey<K>>,
     K: Ord + Clone + core::fmt::Debug,
 {
     fn max_key(&'a self) -> Result<Option<Branch<'a, Self, A>>, CanonError> {
@@ -192,7 +192,6 @@ where
         &'a mut self,
     ) -> Result<Option<BranchMut<'a, Self, A>>, CanonError>
     where
-        A: Combine<Self, A>,
         C: MutableLeaves,
     {
         // Return the first mutable branch that satisfies the walk
